import unittest
from datetime import datetime

from mock import patch
from mockredis import mock_strict_redis_client
from mockredis.exceptions import RedisError
import mock

from redisEventTracker import EventTracker


class TestConnection(unittest.TestCase):
    @patch('redisEventTracker.StrictRedis', mock_strict_redis_client)
    def setUp(self):
        self.et = EventTracker()
        self.date = datetime.now().date()

    def test_check_date(self):
        self.assertEqual(self.et._redis.sismember('dates', self.date), 0, u'Key is exist, or element member of set')
        self.assertEqual(self.et._redis.sadd('dates', self.date), 1, u'Key must be added here')
        self.assertEqual(self.et._redis.sadd('dates', self.date), 0, u'Key must be already exist')

    def test_add_event(self):
        self.et.track_event('event')
        self.assertEqual(self.et._redis.hget('event', self.date), '1')

    def test_inc_event(self):
        self.et.track_event('event')
        self.et.track_event('event')
        self.assertEqual(self.et._redis.hget('event', self.date), '2')

    @patch('redisEventTracker.logger')
    def test_logging(self, mocked_logger):
        self.et._redis.sadd = mock.Mock(side_effect=RedisError('Boom!'))
        self.et.track_event('event')
        self.assertTrue(mocked_logger.warning.called)

    def test_tracker_param(self):
        self.assertRaises(TypeError, self.et.track_event)


# encoding: utf-8

from redis import StrictRedis
from redis.exceptions import RedisError
from datetime import datetime
import warnings
import logging
import socket
import time


logger = logging.getLogger(__name__)
logger.addHandler(logging.NullHandler())


class Graphite(object):
    def __init__(self, host='localhost', port=2003, prefix=''):
        self.socket = socket.socket()
        self.socket.connect((host, port))
        self.prefix = 'event_tracker.'
        if prefix:
            self.prefix += prefix + '.'

    def send_metric(self, metric_name, msg):
        metric_name = self.prefix + metric_name
        message = '%s %s %d\n' % (metric_name, msg, int(time.time()))
        self.socket.sendall(message)


class Singleton(object):
    _instance = None

    def __new__(cls, *args, **kwargs):
        if not cls._instance:
            cls._instance = super(Singleton, cls).__new__(cls, *args, **kwargs)
        return cls._instance


class EventTracker(Singleton):
    _redis = None

    def __init__(self, redis=None, host='localhost', port=6379, db=0, graphite_host=None, graphite_port=2003,
                 graphite_prefix=''):
        self.set_connection_to_redis(redis or self.get_connection_to_redis(host=host, port=port, db=db))
        self.graphite_host = graphite_host
        self.graphite_port = graphite_port
        self.graphite = None
        if graphite_host:
            try:
                self.graphite = Graphite(graphite_host, graphite_port, graphite_prefix)
            except Exception as e:
                msg = u"could not connect to graphite server: %s" % unicode(e)
                warnings.warn(msg)
                logger.warning(msg)

    @staticmethod
    def get_connection_to_redis(**kwargs):
        return StrictRedis(**kwargs)

    def set_connection_to_redis(self, redis):
        self._redis = redis

    def track_event(self, event_hash_name):
        date = datetime.now().date()
        try:
            if not self._redis.sismember('dates', date):
                self._redis.sadd('dates', date)
            total = self._redis.hincrby(event_hash_name, date, 1)

            if self.graphite:
                self.graphite.send_metric(event_hash_name, total)

        except RedisError as e:
            warnings.warn(unicode(e))
            logger.warning(u'{0}; event: {1}'.format(unicode(e), event_hash_name))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os

from setuptools import setup, find_packages
try:
  readme = open(os.path.join(os.path.dirname(__file__), 'README.rst')).read()
except:
  readme = ''

setup(
    name='redis-event-tracker',
    version='0.1.0',
    packages=find_packages(),
    requires=['redis (>= 2.7.6)'],
    description='Track your events in redis',
    long_description=readme,
    author='Keyintegrity',
    author_email='info@keyintegrity.com',
    maintainer='Aleksey Pivovarov',
    maintainer_email='lexapi@gmail.com',
    url='https://github.com/Keyintegrity/redisEventTracker',
    download_url='https://github.com/Keyintegrity/redisEventTracker/archive/master.zip',
    license='MIT License',
    keywords='redis events track',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'Intended Audience :: System Administrators',
        'Programming Language :: Python :: 2.7',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)